﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;


// 1. reference Assemblies, Framework: System.windows.Controls.Ribbon
//      Ordner: Assemblies, Framework    suchen Ribbon
// 2. using System.Windows.Controls.Ribbon
using System.Windows.Controls.Ribbon;
using Microsoft.Win32;
using System.IO;
using System.Threading;
using System.ComponentModel;
using System.Collections;
using System.Diagnostics;
using Backup.Model;
using Backup.FileOperations;
using Backup.NormalWindows;
using Backup.RunWindows;

namespace Backup {
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : RibbonWindow {


        private ModelProject project = new ModelProject();

        private Options options = Options.getInstance();

        private bool start = false;

        public MainWindow() {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e) {
            int version1 = Basis.ReadRegIntLocal_Maschine("SOFTWARE\\Microsoft\\NET Framework Setup\\NDP\\v4\\Full", "Release");
            if (version1 == 0) {
                Basis.ErrorMsg("Bitte installieren Sie die .net Version 4.5 oder höher", "Fehler");
            }
            else {
                if (version1 < 378389) {
                    Basis.ErrorMsg("Bitte installieren Sie die .net Version 4.5 oder höher", "Fehler");
                    Close();
                }
                else {
                    String version2 = Basis.ReadRegStringLocal_Maschine("SOFTWARE\\Microsoft\\NET Framework Setup\\NDP\\v4\\Full", "Version");
                    // editor.Text = "Die nötige Framework-Version ist vorhanden\r\n  Versionszahl: " + version1 + "\r\n  Version: " + version2;
                }
            }

            String lastProject = options.BackupLastProject;
            if (File.Exists(lastProject)) {
                tLastProjectFile.Text = Basis.getFilename(lastProject);
                tLastProjectFile.ToolTip = lastProject;
                bnOpenLastProject.IsEnabled = true;
            }
            else {
                bnOpenLastProject.IsEnabled = false;
            }

            setGUI();

        }  // loaded

        private void setGUI() {
            // set the position und size of the window
            int iTop = Basis.ReadRegInt(MWRegistry.C_REG_ADR, "iTop1");
            int iLeft = Basis.ReadRegInt(MWRegistry.C_REG_ADR, "iLeft1");
            int iWidth = Basis.ReadRegInt(MWRegistry.C_REG_ADR, "iWidth1");
            int iHeight = Basis.ReadRegInt(MWRegistry.C_REG_ADR, "iHeight1");
            int w = (int)System.Windows.SystemParameters.PrimaryScreenWidth;
            int h = (int)System.Windows.SystemParameters.PrimaryScreenHeight;
            if (iTop > 0 && iLeft > 0 && iLeft < w && iWidth > 50) {
                Top = iTop;
                Left = iLeft;
                Width = iWidth;
                Height = iHeight;
            }
            else {
                Top = 100;
                Left = 20;
            }
            int iVerify = Basis.ReadRegInt(MWRegistry.C_REG_ADR, "Verify");
            chkVerify.IsChecked = iVerify == 1;

            CommandBindings.Add(new CommandBinding(MyCommands.OpenProject, OpenProjectExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.NewProject, NewProjectExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.SaveProject, SaveProjectExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.SaveAsProject, SaveAsProjectExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.RunProject, RunProjectExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.RunProjectFull, RunProjectFullExecuted, CheckCanExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.InsertDir, InsertDirExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.DeleteSelectionItems, DeleteSelectionItemsExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.SelectNone, SelectNoneExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.InvertSelection, InvertSelectionExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.SelectAll, SelectAllExecuted, CheckCanExecuted));


            CommandBindings.Add(new CommandBinding(MyCommands.RunExplorer, RunExplorerExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.DeleteLastProjects, DeleteLastProjectsExecuted));



        } // setGUI


        private void CheckCanExecuted(object sender, CanExecuteRoutedEventArgs e) {
            e.CanExecute = project.Filename.Length > 0;
        }


        private void DeleteLastProjectsExecuted(object sender, ExecutedRoutedEventArgs e) {
            start = true;
            options.BackupLastProject = "";
            bnOpenLastProject.IsEnabled = false;
            tLastProjectFile.Text = "";
            tDest.Text = "";
            tNumbersFullBackup.Text = "";
            tNumbersBackup.Text = "";
            tDateBackup.Text = "";
            listboxDirs.Items.Clear();
            tNegativeExts.Text = "";
            project.Clear(true);
            project.Filename = "";
            tLastProjectFile.Foreground = Brushes.Black;  
            start = false;
        }

        private void RunExplorerExecuted(object sender, ExecutedRoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Explorer.exe";
            if (project.Dest.Length > 0)
                Prog.StartInfo.Arguments = "/e," + project.Dest;
            else
                Prog.StartInfo.Arguments = "/e,";
            Prog.Start();
        }




        private void SaveProjectExecuted(object sender, ExecutedRoutedEventArgs e) {
            project.Save();
        }


        private void bnOpenLastProject_Click(object sender, RoutedEventArgs e) {
            OpenProject(options.BackupLastProject);
            tLastProjectFile.Foreground = Brushes.Blue;
            chkVerify.IsEnabled = true;
        }

        private void setactiveProjekt(String filename) {
            Title = "Backup: " + filename;
            options.BackupLastProject = filename;
            tLastProjectFile.Text = Basis.getFilename(filename);
            tLastProjectFile.ToolTip = filename;
            tLastProjectFile.Foreground = Brushes.Blue;
            project.Filename = filename;
            project.Save();            
            bnOpenLastProject.IsEnabled = true;
            chkVerify.IsEnabled = true;
        }


        private void SaveAsProjectExecuted(object sender, ExecutedRoutedEventArgs e) {
            String filename = "";
            SaveFileDialog savefiledialog = new SaveFileDialog();
            savefiledialog.Title = "Projekt speichern unter";
            savefiledialog.Filter = "Backup-Projekt (*.prj)|*.prj|Alle Dateien|*.*";
            savefiledialog.DefaultExt = ".prj";
            savefiledialog.FilterIndex = 1; // zählt von 1
            savefiledialog.InitialDirectory = Environment.CurrentDirectory;
            if (savefiledialog.ShowDialog() == true) {
                String filenameOld = project.Filename;
                filename = savefiledialog.FileName;
                project.NumberBackup = 0;
                project.DayLastBackup = 0;
                project.MonthLastBackup = 0;
                project.YearLastBackup = 0;
                tDateBackup.Text = "";
                setactiveProjekt(filename);
            }
        }  // SaveAsProjectExecuted



        private void NewProjectExecuted(object sender, ExecutedRoutedEventArgs e) {
            String filename = "";
            SaveFileDialog savefiledialog = new SaveFileDialog();
            savefiledialog.Title = "Neues Projekt";
            savefiledialog.Filter = "Backup-Projekt (*.prj)|*.prj|Alle Dateien|*.*";
            savefiledialog.DefaultExt = ".prj";
            savefiledialog.FilterIndex = 1; // zählt von 1
            savefiledialog.InitialDirectory = Environment.CurrentDirectory;
            if (savefiledialog.ShowDialog() == true) {
                filename = savefiledialog.FileName;
                tNumbersFullBackup.Text = "10";
                tNumbersBackup.Text = "0";
                tDateBackup.Text = "";
                project.Clear(true);
                project.Filename = filename;
                project.Save();
                chkVerify.IsEnabled = true;
                tLastProjectFile.Text = Basis.getFilename(filename);
                options.BackupLastProject = filename;
            }

        }  // NewProjectExecuted

        private void OpenProjectExecuted(object sender, ExecutedRoutedEventArgs e) {
            OpenFileDialog openfiledialog = new OpenFileDialog();
            openfiledialog.Title = "Laden eines Projekt";
            openfiledialog.Filter = "Backup-Projekt (*.prj)|*.prj|Alle Dateien|*.*";
            openfiledialog.DefaultExt = ".prj";
            openfiledialog.FilterIndex = 1; // zählt von 1
            openfiledialog.InitialDirectory =
                Environment.CurrentDirectory;
            if (openfiledialog.ShowDialog() == true) {
                OpenProject(openfiledialog.FileName);
            }
        }


        private void OpenProject(String filename) {
            if (project.Filename.Length > 0 && project.modifyAttrib) {
                MessageBoxResult res = Basis.readYesNoCancel("Wollen Sie die Änderungen im Projekt speichern?\r\n" +
                    "Projekt: " + Basis.getFilename(project.Filename), "Neues Projekt laden");
                switch (res) {
                    case MessageBoxResult.Cancel: return;
                    case MessageBoxResult.Yes:
                        project.Save();
                        break;
                    case MessageBoxResult.No: break;
                } // switch
            }

            project.Clear(true);
            project.Filename = filename;
            project.Load();
            listboxDirs.Items.Clear();
            tDest.Text = project.Dest;
            tNumbersFullBackup.Text = project.NumbersFullBackup.ToString();
            tNumbersBackup.Text = project.NumberBackup.ToString();
            if (project.YearLastBackup == 0)
                tDateBackup.Text = "";
            else
                tDateBackup.Text = project.DayLastBackup.ToString("00") + "." + project.MonthLastBackup.ToString("00") + "." + project.YearLastBackup.ToString("00");
            for (int i = 0; i < project.CountSources; i++) {
                String dir = project.getDir(i);
                addSource2ListBox(dir);
            }
            start = true;
            tNegativeExts.Text = "";
            for (int i = 0; i < project.CountNegativeExt; i++) {
                String ext = project.getNegativeExt(i);
                tNegativeExts.AppendText(ext + "\r\n");
            }
            start = false;

            if (options.BackupLastProject.Length == 0 || tLastProjectFile.Text.Equals(Basis.getFilename(filename))) {
                tLastProjectFile.Text = Basis.getFilename(filename);
                options.BackupLastProject = filename;
            }
            chkVerify.IsEnabled = true;
        }  // open Project

        private void bnSelectDest_Click(object sender, RoutedEventArgs e) {
            // tDest.Text = project.Dest;
            String path = Basis.ReadRegString(MWRegistry.C_REG_ADR, "backupMainPath");
            if (MWRegistry.BrowserFolderDialogWithoutFiles) {
                WSelectDirectory2 dialog = new WSelectDirectory2(this, path);
                dialog.ShowDialog();
                if (dialog.Retcode) {
                    insertDestDir(dialog.SelectDirectory);
                }
            }
            else {
                WSelectDirectory dialog = new WSelectDirectory(this, path);
                dialog.ShowDialog();
                if (dialog.Retcode) {
                    insertDestDir(dialog.SelectDirectory);
                }

            }
        }

        private void insertDestDir(String dir) {
            options.BackupMainPath = dir;
            tDest.Text = dir;
            project.Dest = dir;
        }


        private void addSource2ListBox(String dir) {
            ListBoxItem li = new ListBoxItem();
            //CheckBox chk = new CheckBox();
            //chk.Content = dir;
            //chk.Tag = dir;
            //chk.Click += new RoutedEventHandler(chk_Click);
            //li.Content = chk;
            li.Content = dir;
            li.Tag = dir;
            listboxDirs.Items.Add(li);
        }

        private void insertBackupDir(String source) {
            if (Basis.readNoYes("Sollen die Unterverzeichnisse eingetragen werden?", "Neue Backup-Verzeichnisse")) {
                DirectoryInfo mainDir = new DirectoryInfo(source);
                DirectoryInfo[] dirs = mainDir.GetDirectories();
                foreach (DirectoryInfo dir in dirs) {
                    if (dir.Name.Equals("$RECYCLE.BIN")) continue;
                    if ((dir.Attributes & FileAttributes.Hidden) == FileAttributes.Hidden) continue;
                    if ((dir.Attributes & FileAttributes.NoScrubData) == FileAttributes.NoScrubData) continue;
                    if ((dir.Attributes & FileAttributes.Offline) == FileAttributes.Offline) continue;
                    if ((dir.Attributes & FileAttributes.ReparsePoint) == FileAttributes.ReparsePoint) continue;
                    if ((dir.Attributes & FileAttributes.System) == FileAttributes.System) continue;
                    if ((dir.Attributes & FileAttributes.Temporary) == FileAttributes.Temporary) continue;
                    addSource2ListBox(dir.FullName);
                    project.addDir(dir.FullName);
                }
            }
            else {
                addSource2ListBox(source);
                project.addDir(source);
            }
        }

        /*  Archive	Compressed	Device	Directory	Encrypted	NoScrubData NotContentIndexed	Offline	 ReadOnly	ReparsePoint	 SparseFile	
                Temporary
         * */



        private void bnProgInfo_Click(object sender, RoutedEventArgs e) {
            (new WProgInfo(this)).ShowDialog();
        }

        private void bnProperties_Click(object sender, RoutedEventArgs e) {
            (new WOptions(this)).ShowDialog();
        }

        private void bnHelp_Click(object sender, RoutedEventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = Basis.getProgPath() + "Handbuch_IGMBackup.pdf";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
            Prog.WaitForExit();
            Prog.Close();
        }



        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e) {
            if (project.Filename.Length > 0 && project.modifyAttrib) {
                MessageBoxResult res = Basis.readYesNoCancel("Wollen Sie die Änderungen im Projekt speichern?\r\n" +
                    "Projekt: " + Basis.getFilename(project.Filename), "Programm schließen");
                switch (res) {
                    case MessageBoxResult.Cancel:
                        e.Cancel = true;
                        break;
                    case MessageBoxResult.Yes:
                        project.Save();
                        break;
                    case MessageBoxResult.No: break;
                } // switch
            }
            // save the position and the size in the registry
            Basis.WriteRegInt(MWRegistry.C_REG_ADR,
            "iTop1", (int)this.Top);
            Basis.WriteRegInt(MWRegistry.C_REG_ADR,
            "iLeft1", (int)this.Left);
            Basis.WriteRegInt(MWRegistry.C_REG_ADR,
            "iWidth1", (int)this.Width);
            Basis.WriteRegInt(MWRegistry.C_REG_ADR,
            "iHeight1", (int)this.Height);
        }




        private void SelectAllExecuted(object sender, ExecutedRoutedEventArgs e) {
            foreach (ListBoxItem li in listboxDirs.Items) {
                //CheckBox chk = (CheckBox)li.Content;
                //chk.IsChecked=true;
                li.IsSelected = true;
            }
        }

        private void InvertSelectionExecuted(object sender, ExecutedRoutedEventArgs e) {
            foreach (ListBoxItem li in listboxDirs.Items) {
                //CheckBox chk = (CheckBox)li.Content;
                if (li.IsSelected == true)
                    li.IsSelected = false;
                else
                    li.IsSelected = true;
            }
        }

        private void SelectNoneExecuted(object sender, ExecutedRoutedEventArgs e) {
            foreach (ListBoxItem li in listboxDirs.Items) {
                //CheckBox chk = (CheckBox)li.Content;
                li.IsSelected = false;
            }
        }

        private void DeleteSelectionItemsExecuted(object sender, ExecutedRoutedEventArgs e) {
            //Rückwärts
            for (int i = listboxDirs.Items.Count - 1; i >= 0; i--) {
                ListBoxItem li = (ListBoxItem)listboxDirs.Items[i];
                //CheckBox chk = (CheckBox)li.Content;
                if (li.IsSelected == true) {
                    listboxDirs.Items.RemoveAt(i);
                    project.removeDir(i);
                }
            }
        }

        private void InsertDirExecuted(object sender, ExecutedRoutedEventArgs e) {
            String path = Basis.ReadRegString(MWRegistry.C_REG_ADR, "backupSourcePath");
            if (MWRegistry.BrowserFolderDialogWithoutFiles) {
                WSelectDirectory2 dialog = new WSelectDirectory2(this, path);
                dialog.ShowDialog();
                if (dialog.Retcode) {
                    insertBackupDir(dialog.SelectDirectory);
                }
            }
            else {
                WSelectDirectory dialog = new WSelectDirectory(this, path);
                dialog.ShowDialog();
                if (dialog.Retcode) {
                    insertBackupDir(dialog.SelectDirectory);
                }

            }
        }


        private void RunProjectExecuted(object sender, ExecutedRoutedEventArgs e) {
            RunProject(false);
        }
        private void RunProjectFullExecuted(object sender, ExecutedRoutedEventArgs e) {
            RunProject(true);
        }


        private void RunProject(bool full) {
            if (!Directory.Exists(project.Dest)) {
                Basis.ErrorMsg("Das Zielverzeichnis ist nicht da\r\n" + project.Dest, "Fehler");
                return;
            }
            if ((project.NumberBackup == project.NumbersFullBackup) || (project.YearLastBackup == 0) || full)
                CopyDirectory.LastBackup = new DateTime(1900, 1, 1);
            else
                CopyDirectory.LastBackup = new DateTime(project.YearLastBackup, project.MonthLastBackup, project.DayLastBackup);

            CopyDirectory.negativeExt.Clear();
            for (int i = 0; i < project.CountNegativeExt; i++) {
                CopyDirectory.negativeExt.Add(project.getNegativeExt(i).ToLower());
            }

            WRunBackup dialog = new WRunBackup(project, chkVerify.IsChecked == true);
            dialog.ShowDialog();
            Title = "Backup: " + project.Filename + "     Zeit: " + dialog.StopwatchTime;
            if (dialog.Error.Length > 0) {
                File.WriteAllText(Basis.getProgPath() + "error.txt", "Es sind Fehler beim Kopieren aufgetreten\r\n\r\n" + dialog.Error);
                Process Prog = new Process();
                Prog.StartInfo.FileName = "Notepad.exe";
                Prog.StartInfo.Arguments = Basis.getProgPath() + "error.txt";
                Prog.Start();
            }
            project.DayLastBackup = System.DateTime.Now.Day;
            project.MonthLastBackup = System.DateTime.Now.Month;
            project.YearLastBackup = System.DateTime.Now.Year;
            if (project.NumberBackup == project.NumbersFullBackup)
                project.NumberBackup = 0;
            else
                project.NumberBackup = project.NumberBackup + 1;
            tNumbersBackup.Text = project.NumberBackup.ToString();
            project.Save();
            tDateBackup.Text = project.DayLastBackup.ToString("00") + "." + project.MonthLastBackup.ToString("00") + "." + project.YearLastBackup.ToString("00");

        }

        private void tNegativeExts_TextChanged(object sender, TextChangedEventArgs e) {
            if ((!start) && project.Filename.Length > 0) {
                project.addtext2NegativeExt(tNegativeExts.Text.Trim());
            }
        }

        private void chkVerify_Click(object sender, RoutedEventArgs e) {
            if (chkVerify.IsChecked == true)
                Basis.WriteRegInt(MWRegistry.C_REG_ADR, "Verify", 1);
            else
                Basis.WriteRegInt(MWRegistry.C_REG_ADR, "Verify", 0);
        }

      


    }

}
