﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.IO;


namespace Backup.Model {
    public class ModelProject : ModelBase {

        private String VERSION = "1";

        private String filename = "";
        public String Filename {
            get { return filename; }
            set {
                filename = value;
                base.modifyAttrib = true;
            }
        }

        private int numbersFullBackup = 10;
        public int NumbersFullBackup {
            get { return numbersFullBackup; }
            set {
                numbersFullBackup = value;
                base.modifyAttrib = true;
            }
        }



        private int numberBackup = 0;
        public int NumberBackup {
            get { return numberBackup; }
            set {
                numberBackup = value;
                base.modifyAttrib = true;
            }
        }


        private int yearLastBackup = 0;
        public int YearLastBackup {
            get { return yearLastBackup; }
            set {
                yearLastBackup = value;
            }
        }
        private int monthLastBackup = 0;
        public int MonthLastBackup {
            get { return monthLastBackup; }
            set {
                monthLastBackup = value;
            }
        }
        private int dayLastBackup = 0;
        public int DayLastBackup {
            get { return dayLastBackup; }
            set {
                dayLastBackup = value;
            }
        }


        private String dest = "";
        public String Dest {
            get { return dest; }
            set {
                dest = value;
                base.modifyAttrib = true;
            }
        }
        public String DestDateTime {
            get {
                String destDir;
                if (numberBackup == numbersFullBackup) {
                    if (dest.EndsWith("\\"))
                        destDir = dest + Basis.getDatumBackup() + "-Full\\";
                    else
                        destDir = dest + "\\" + Basis.getDatumBackup() + "-Full\\";
                }
                else {
                    if (dest.EndsWith("\\"))
                        destDir = dest + Basis.getDatumBackup() + "\\";
                    else
                        destDir = dest + "\\" + Basis.getDatumBackup() + "\\";
                }
                return destDir;
            }
        }

        private List<String> sources = new List<string>();
        private List<String> negativeExt = new List<string>();

        internal int CountSources {
            get { return sources.Count; }
        }

        internal void addDir(String dir) {
            if (Directory.Exists(dir)) {
                sources.Add(dir);
                base.modifyAttrib = true;
            }
            else {
                Basis.ErrorMsg("Das Verzeichnis ist nicht vorhanden\r\n" + dir, "Fehler");
            }
        }

        internal void removeDir(int i) {
            if (i >= 0 && i < sources.Count) {
                sources.RemoveAt(i);
                base.modifyAttrib = true;
            }
        }

        internal String getDir(int i) {
            if (i >= 0 && i < sources.Count) {
                return sources[i];
            }
            else {
                return "";
            }
        }


        public void addtext2NegativeExt(String text) {
            negativeExt.Clear();
            String[] items = Basis.getLines(text);
            foreach (String item in items) {
                if (item.Length > 0) {
                    negativeExt.Add(item);
                }
            }
            // hier kein Speichern, sonst rekursion
        }

        internal int CountNegativeExt {
            get { return negativeExt.Count; }
        }

        internal void addNegativeExts(String editor) {
        }

        internal String getNegativeExt(int i) {
            if (i >= 0 && i < negativeExt.Count) {
                return negativeExt[i];
            }
            else {
                return "";
            }
        }

        internal void Clear(bool deleteFilename) {
            dest = "";
            if (deleteFilename) filename = "";
            sources.Clear();
            negativeExt.Clear();
            base.modifyAttrib = false;
        }

        internal void Load() {
            if (File.Exists(filename)) {
                Clear(false);
                FileStream inFile = new FileStream(filename, FileMode.Open);
                StreamReader inStream = new StreamReader(inFile);

                String version = inStream.ReadLine();
                if (!version.Equals(VERSION)) {
                    Basis.ErrorMsg("Fehlerhafte Version\r\n" + version, "Fehler");
                    inStream.Close();
                    inFile.Close();
                    return;
                }


                int n;
                String sN;
                sN = inStream.ReadLine();  // 
                if (!int.TryParse(sN, out n)) {
                    Basis.ErrorMsg("Fehlerhafte Anzahl des normalen Kopierens\r\n" + sN, "Fehler");
                    inStream.Close();
                    inFile.Close();
                    return;
                }
                if (n < 0) {
                    Basis.ErrorMsg("Fehlerhafte Anzahl des normalen Kopierens\r\n" + sN, "Fehler");
                    inStream.Close();
                    inFile.Close();
                    return;
                }
                numbersFullBackup = n;


                sN = inStream.ReadLine();  // 
                if (!int.TryParse(sN, out numberBackup)) {
                    numberBackup = 0;
                }

                sN = inStream.ReadLine();  // 
                if (!int.TryParse(sN, out dayLastBackup)) {
                    dayLastBackup = 0;
                }


                sN = inStream.ReadLine();  // 
                if (!int.TryParse(sN, out monthLastBackup)) {
                    monthLastBackup = 0;
                }

                sN = inStream.ReadLine();  // 
                if (!int.TryParse(sN, out yearLastBackup)) {
                    yearLastBackup = 0;
                }



                dest = inStream.ReadLine();

                sN = inStream.ReadLine();
                if (!int.TryParse(sN, out n)) {
                    Basis.ErrorMsg("Fehlerhafte Anzahl der Quellen-Verzeichnisse\r\n" + sN, "Fehler");
                    inStream.Close();
                    inFile.Close();
                    return;
                }
                if (n < 0) {
                    Basis.ErrorMsg("Fehlerhafte Anzahl der Quellen-Verzeichnisse\r\n" + sN, "Fehler");
                    inStream.Close();
                    inFile.Close();
                    return;
                }
                String dir;
                for (int i = 0; i < n; i++) {
                    dir = inStream.ReadLine();
                    addDir(dir);
                }


                sN = inStream.ReadLine();
                if (!int.TryParse(sN, out n)) {
                    Basis.ErrorMsg("Fehlerhafte Anzahl der fehlernaften Erweiterung\r\n" + sN, "Fehler");
                    inStream.Close();
                    inFile.Close();
                    return;
                }
                if (n < 0) {
                    Basis.ErrorMsg("Fehlerhafte Anzahl der fehlernaften Erweiterung\r\n" + sN, "Fehler");
                    inStream.Close();
                    inFile.Close();
                    return;
                }
                String ext;
                for (int i = 0; i < n; i++) {
                    ext = inStream.ReadLine();
                    negativeExt.Add(ext);
                }

                inStream.Close();
                inFile.Close();
                base.modifyAttrib = false;
            }
            else {
                Basis.ErrorMsg("Die Projektdatei ist nicht vorhanden\r\n" + filename, "Fehler");
            }
        }

        internal void Save() {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("1");  // Version
            sb.AppendLine(numbersFullBackup.ToString());


            sb.AppendLine(numberBackup.ToString());
            sb.AppendLine(dayLastBackup.ToString());
            sb.AppendLine(monthLastBackup.ToString());
            sb.AppendLine(yearLastBackup.ToString());

            sb.AppendLine(dest);

            sb.AppendLine(sources.Count.ToString());
            foreach (String dir in sources) {
                sb.AppendLine(dir);
            }

            sb.AppendLine(negativeExt.Count.ToString());
            foreach (String ext in negativeExt) {
                sb.AppendLine(ext);
            }


            sb.AppendLine();
            File.WriteAllText(filename, sb.ToString());
            base.modifyAttrib = false;
        }

    }
}
