﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using System.ComponentModel;
using System.IO;
using System.Threading;
using System.Diagnostics;
using System.Security.Cryptography;
using Backup.FileOperations;
using Backup.Model;

namespace Backup.RunWindows {
    /// <summary>
    /// Interaction logic for WRunBackup.xaml
    /// </summary>
    public partial class WRunBackup : Window {


        public String Error {
            get { return sbError.ToString(); }
        }
        public String StopwatchTime {
            get { return stopwatch.Elapsed.ToString(); }
        }

        private BackgroundWorker worker = new BackgroundWorker();
        private Stopwatch stopwatch = new Stopwatch();

        ModelProject project = null;
        bool verify = false;

        public WRunBackup(ModelProject project, bool verify) {
            InitializeComponent();
            this.project = project;
            this.verify = verify;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e) {
            this.Cursor = Cursors.Wait;
            calcSizeofbackup();
            this.Cursor = Cursors.Arrow;
            labelCaption.Content = "Backup: " + Basis.getFilename(project.Filename);
            initBackup();
        }



        private void initBackup() {
            ende1 = false;
            ende2 = false;
            editorMainProgress.Text = "";
            editorRemark.Text = "";
            worker.WorkerReportsProgress = true;
            worker.ProgressChanged += worker_ProgressChangedTest;
            int n = project.CountSources;

            destDir = project.DestDateTime;

            progressbar1.Maximum = n + 1;  // add one number, because this methode execute twice
            worker.DoWork += startBackup;
            worker.RunWorkerAsync();
            stopwatch.Start();

            Thread t1 = new Thread(calcSizeofbackup);
            t1.Start();
        }


        public void calcSizeofbackup() {
            long summe = 0;
            long numbersOfFiles = 0;
            for (int i = 0; i < project.CountSources; i++) {
                String source = project.getDir(i);  // c:\eclipse
                DirectoryInfo mainDir = new DirectoryInfo(source);

                FileInfo[] files = mainDir.GetFiles("*.*", SearchOption.AllDirectories);
                numbersOfFiles += files.Length;
                mainDir = null;
                foreach (FileInfo file in files) {
                    summe += file.Length;
                }
                files = null;
            }
            maxBytes = summe;
            maxFiles = numbersOfFiles;
            // progressbar2.Maximum = maxsize;
        }

        public string checkMD5(string filename) {
            using (var md5 = MD5.Create()) {
                using (var stream = File.OpenRead(filename)) {
                    return Encoding.Default.GetString(md5.ComputeHash(stream));
                }
            }
        }


        private void startBackup(object sender, DoWorkEventArgs e) {
            int n = 0;

            for (int i = 0; i < project.CountSources; i++) {
                if (abbruchdurchButton) {
                    ende1 = true;
                    worker.ReportProgress(++n);
                    return;
                }
                String source = project.getDir(i);  // c:\eclipse
                String lastDir = Basis.getLastFolder(source);
                String dest = destDir;  
                dest += lastDir;
                progressDir = source;
                isprogressDir = true;

                CopyDirectory copyDir = new CopyDirectory(source, dest);
                // erzeugt die kopierliste
                if (copyDir.StartCopyFiles1()) {

                    maxlocalFiles = copyDir.listCopy.Count;
                    indexlocalFiles = 0;
                    if (copyDir.listCopy.Count > 0) {
                        foreach (ModelCopy copy in copyDir.listCopy) {
                            indexlocalFiles++;
                            progressFile1 = Basis.getFilename(copy.source);
                            filesize = copy.length;
                            worker.ReportProgress(++n);
                            try {
                                File.Copy(copy.source, copy.dest, true);  // true = overwrite
                                if (verify) {
                                    String md5Source = checkMD5(copy.source);
                                    String md5Dest = checkMD5(copy.dest);
                                    if (!md5Source.Equals(md5Dest)) {
                                        Basis.ErrorMsg("Fehlerhaftes kopieren\r\n" +
                                            "Quelle: " + copy.source + "\r\n" +
                                            "Quelle: " + copy.dest, "Fehlerhaftes kopieren");
                                    }
                                }

                                size += copy.length;
                                //progressFile2 = "    ok";
                                //worker.ReportProgress(++n);
                                isprogressDir = false;
                                if (abbruchdurchButton) {
                                    ende1 = true;
                                    worker.ReportProgress(++n);
                                    return;
                                }
                            }
                            catch (Exception excpt) {
                                errorText = copy.source + "\r\n" + excpt.ToString();
                                sbError.AppendLine(errorText);
                                progressFile2 = "    ***** FEHLER *****";
                                worker.ReportProgress(++n);
                                isprogressDir = false;
                                if (abbruchdurchButton) {
                                    ende1 = true;
                                    worker.ReportProgress(++n);
                                    return;
                                }
                            }


                        }
                    }
                    else {
                        Directory.Delete(dest, true);
                    }
                }
            }
            ende1 = true;
            worker.ReportProgress(++n);  // last message!
        }

        bool abbruchdurchButton = false;
        String destDir = "";
        private bool ende1 = false;
        private bool ende2 = false;
        private bool isprogressDir = false;
        private String errorText = "";
        private String progressDir = "";
        private String progressFile1 = "";
        private String progressFile2 = "";
        private long size = 0;
        private long maxBytes = 0;
        private long maxFiles = 0;
        private long indexlocalFiles = 0;
        private long maxlocalFiles = 0;
        private long filesize = 0;
        private bool progressbar2MaxEmpty = true;
        private StringBuilder sbError = new StringBuilder();

        // Wichtig, diese Methode wird nach jeder Änderung aufgerufen.
        // Die Abfrage "istEnde" dient als EndeAktion
        private void worker_ProgressChangedTest(object sender, ProgressChangedEventArgs e) {
            if (progressbar2MaxEmpty && maxBytes > 0) {
                progressbar1.Maximum = maxFiles;
                progressbar2.Maximum = maxBytes;
                progressbar2MaxEmpty = false;
            }
            progressbar1.Value = e.ProgressPercentage;
            labelFortschritt.Content = "Fortschritt: " + stopwatch.Elapsed;

            progressbar2.Value = size;
            if (errorText.Length > 0) {
                editorRemark.AppendText(errorText + "\r\n\r\n");
                errorText = "";
            }



            if (isprogressDir) {
                this.tActualDir.Text = progressDir;
                editorMainProgress.AppendText(progressDir + "\r\n");
                isprogressDir = false;
            }
            if (progressFile1.Length > 0) {
                tActualFile.Text = progressFile1;
                tFileSize.Text = Basis.Best_KB(filesize);
                progressFile1 = "";
                tTotalCount.Text = maxlocalFiles.ToString();
                tCountCopy.Text = indexlocalFiles.ToString();
            }
            if (progressFile2.Length > 0) {
                this.tActualFile.AppendText(progressFile2);
                progressFile2 = "";
            }
            if (ende1) {
                if (!ende2) {
                    ende2 = true;
                    stopwatch.Stop();
                    progressbar1.Value = 0;
                    progressbar2.Value = 0;
                    Close();
                }
            }
        } // worker_ProgressChanged1

        private void bnCancel_Click(object sender, RoutedEventArgs e) {
            abbruchdurchButton = true;
        }



    }
}
